
# DateTags.pm
# Module containing common routines for DateTags plugin package
# by Kevin Shay
# http://www.staggernation.com/mtplugins/DateTagsReadMe.html
# DateTags version 2.3
# last modified 7/9/03

use strict;
package DateTags;
use Exporter;
@DateTags::ISA = qw( Exporter );
use vars qw( @EXPORT );
@EXPORT = qw( dt_get_ts dt_build_date dt_build_value dt_nwomy );
use Date::Calc qw( Decode_Date_EU Decode_Date_US Nth_Weekday_of_Month_Year );

sub dt_get_ts {
# given a context, return a timestamp
# either from a "date" argument, the current_timestamp, or the MTDate time
	my ($ctx, $args) = @_;
	my $ts;
	if ($args->{'date'}) {
		return dt_build_date($ctx, $args, $args->{'date'});
	} elsif ($ts = $ctx->{'current_timestamp'}) {
		return $ts;
	} else {
		$args->{'format'} = '%Y%m%d%H%M%S';
		return MT::Template::Context::_hdlr_sys_date($ctx, $args);
	}
	
}

sub dt_build_date {
	my ($ctx, $args, $date) = @_;
	$date = dt_build_value($ctx, $date);
	my $decode = defined($args->{'eu'}) ? \&Decode_Date_EU : \&Decode_Date_US;
	if (my ($y, $m, $d) = &$decode($date)) {
		return sprintf("%04d%02d%02d000000", $y, $m, $d);
	} else {
		return $ctx->error('Invalid date passed');
	}
}

sub dt_build_value {
# convert and build MT template tags within a passed value.
	my ($ctx, $value) = @_;
		# within a value argument, you can use MT tags, but with
		# square brackets instead of angle brackets and single quotes
		# instead of double quotes; literal square brackets and single
		# quotes must be escaped with a backslash
		# convert non-escaped []'
	$value =~ s/(?<!\\)\[/</g;
	$value =~ s/(?<!\\)\]/>/g;
	$value =~ s/(?<!\\)'/"/g;
		# de-escape escaped []'
	$value =~ s/\\([\[\]'])/$1/g;
		# any MT tags?
	if ($value =~ /<MT/) {
		my $builder = $ctx->stash('builder');
		my $tok = $builder->compile($ctx, $value);
		$value = $builder->build($ctx, $tok);
		return $ctx->error($builder->errstr) unless defined($value);
	}
	return $value;
}

sub dt_nwomy {
# wrapper for Date::Calc::Nth_Weekday_of_Month_Year
# takes parameters in the order suggested by the function name;
# allows -1 for N, meaning "last," which may be either 5th or 4th;
# returns only one value, the day of the month
	my ($n, $dow, $m, $y) = @_;
		# ensure N is an integer (-1,1..5) to avoid build-time error
	return 0 if (($n !~ /^-?\d+$/) || ($n < -1) || ($n == 0) || ($n > 5));
	my $d;
	if ($n == -1) { 
			# "last" weekday of month - if 5th doesn't exist, use 4th
		if (!((undef, undef, $d) = Nth_Weekday_of_Month_Year($y, $m, $dow, 5))) {
			(undef, undef, $d) = Nth_Weekday_of_Month_Year($y, $m, $dow, 4);
		}
	} else {
		(undef, undef, $d) = Nth_Weekday_of_Month_Year($y, $m, $dow, $n);
	}
	return $d;
}

sub dt_err {
	my ($ctx) = @_;
	return $ctx->error("Couldn't load Date::Calc module");
}

1;
